@@ -233,6 +233,7 @@ avifBool avifPNGRead(const char * inputFilename,
                      avifBool ignoreExif,
                      avifBool ignoreXMP,
                      avifBool allowChangingCicp,
+                     uint32_t imageSizeLimit,
                      uint32_t * outPNGDepth)
 {
     volatile avifBool readResult = AVIF_FALSE;
@@ -322,13 +323,15 @@ avifBool avifPNGRead(const char * inputFilename,
         goto cleanup;
     }
     const avifBool useYCgCoR = (avif->matrixCoefficients == AVIF_MATRIX_COEFFICIENTS_YCGCO_RE);
-#else
-    const avifBool useYCgCoR = AVIF_FALSE;
 #endif
     if (avif->yuvFormat == AVIF_PIXEL_FORMAT_NONE) {
         if ((rawColorType == PNG_COLOR_TYPE_GRAY) || (rawColorType == PNG_COLOR_TYPE_GRAY_ALPHA)) {
             avif->yuvFormat = AVIF_PIXEL_FORMAT_YUV400;
-        } else if (avif->matrixCoefficients == AVIF_MATRIX_COEFFICIENTS_IDENTITY || useYCgCoR) {
+        } else if (avif->matrixCoefficients == AVIF_MATRIX_COEFFICIENTS_IDENTITY
+#if defined(AVIF_ENABLE_EXPERIMENTAL_YCGCO_R)
+                   || useYCgCoR
+#endif
+        ) {
             // Identity and YCgCo-R are only valid with YUV444.
             avif->yuvFormat = AVIF_PIXEL_FORMAT_YUV444;
         } else {
@@ -377,7 +380,7 @@ avifBool avifPNGRead(const char * inputFilename,
         } else if (allowChangingCicp) {
             if (png_get_sRGB(png, info, &srgbIntent) == PNG_INFO_sRGB) {
                 // srgbIntent ignored
-                avif->colorPrimaries = AVIF_COLOR_PRIMARIES_BT709;
+                avif->colorPrimaries = AVIF_COLOR_PRIMARIES_SRGB;
                 avif->transferCharacteristics = AVIF_TRANSFER_CHARACTERISTICS_SRGB;
             } else {
                 avifBool needToGenerateICC = AVIF_FALSE;
@@ -453,6 +456,10 @@ avifBool avifPNGRead(const char * inputFilename,
         fprintf(stderr, "png_get_channels() should return 3 or 4 but returns %d.\n", numChannels);
         goto cleanup;
     }
+    if (avif->width > imageSizeLimit / avif->height) {
+        fprintf(stderr, "Too big PNG dimensions (%u x %u > %u px): %s\n", avif->width, avif->height, imageSizeLimit, inputFilename);
+        goto cleanup;
+    }
 
     avifRGBImageSetDefaults(&rgb, avif);
     rgb.chromaDownsampling = chromaDownsampling;
@@ -473,8 +480,14 @@ avifBool avifPNGRead(const char * inputFilename,
         goto cleanup;
     }
     rowPointers = (png_bytep *)malloc(sizeof(png_bytep) * rgb.height);
+    if (rowPointers == NULL) {
+        fprintf(stderr, "avifPNGRead internal error: memory allocation failure");
+        goto cleanup;
+    }
+    uint8_t * rgbRow = rgb.pixels;
     for (uint32_t y = 0; y < rgb.height; ++y) {
-        rowPointers[y] = &rgb.pixels[y * rgb.rowBytes];
+        rowPointers[y] = rgbRow;
+        rgbRow += rgb.rowBytes;
     }
     png_read_image(png, rowPointers);
     if (avifImageRGBToYUV(avif, &rgb) != AVIF_RESULT_OK) {
@@ -615,7 +628,7 @@ avifBool avifPNGWrite(const char * outputFilename, const avifImage * avif, uint3
         // then they could be written in cHRM/gAMA chunks.
         png_set_iCCP(png, info, "libavif", 0, avif->icc.data, (png_uint_32)avif->icc.size);
     } else {
-        const avifBool isSrgb = (avif->colorPrimaries == AVIF_COLOR_PRIMARIES_BT709) &&
+        const avifBool isSrgb = (avif->colorPrimaries == AVIF_COLOR_PRIMARIES_SRGB) &&
                                 (avif->transferCharacteristics == AVIF_TRANSFER_CHARACTERISTICS_SRGB);
         if (isSrgb) {
             png_set_sRGB_gAMA_and_cHRM(png, info, PNG_sRGB_INTENT_PERCEPTUAL);
@@ -697,16 +710,22 @@ avifBool avifPNGWrite(const char * outputFilename, const avifImage * avif, uint3
     }
 
     rowPointers = (png_bytep *)malloc(sizeof(png_bytep) * avif->height);
+    if (rowPointers == NULL) {
+        fprintf(stderr, "Error writing PNG: memory allocation failure");
+        goto cleanup;
+    }
+    uint8_t * row;
+    uint32_t rowBytes;
     if (monochrome8bit) {
-        uint8_t * yPlane = avif->yuvPlanes[AVIF_CHAN_Y];
-        uint32_t yRowBytes = avif->yuvRowBytes[AVIF_CHAN_Y];
-        for (uint32_t y = 0; y < avif->height; ++y) {
-            rowPointers[y] = &yPlane[y * yRowBytes];
-        }
+        row = avif->yuvPlanes[AVIF_CHAN_Y];
+        rowBytes = avif->yuvRowBytes[AVIF_CHAN_Y];
     } else {
-        for (uint32_t y = 0; y < avif->height; ++y) {
-            rowPointers[y] = &rgb.pixels[y * rgb.rowBytes];
-        }
+        row = rgb.pixels;
+        rowBytes = rgb.rowBytes;
+    }
+    for (uint32_t y = 0; y < avif->height; ++y) {
+        rowPointers[y] = row;
+        row += rowBytes;
     }
     if (avifImageGetExifOrientationFromIrotImir(avif) != 1) {
         // TODO(yguyon): Rotate the samples.
